'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var mergeAll = _interopDefault(require('ramda/src/mergeAll'));
var merge = _interopDefault(require('ramda/src/merge'));
var uncurryN = _interopDefault(require('ramda/src/uncurryN'));
var flip = _interopDefault(require('ramda/src/flip'));
var effects = require('redux-saga/effects');
var redux = require('redux');
var reactRedux = require('react-redux');
var applySpec = _interopDefault(require('ramda/src/applySpec'));
var mapObjIndexed = _interopDefault(require('ramda/src/mapObjIndexed'));
var react = require('react');

/**
 * Creates a redux standard reducer for a state `TState` from an `ActionHandlerMap<State>`
 *
 * @param handlers
 * @param initialState
 */

function createReducer(handlers, initialState) {
  var $handlers = Array.isArray(handlers) ? mergeAll(handlers) : handlers;
  return function (state, action) {
    if (state === void 0) {
      state = initialState;
    }

    var actionReducer = $handlers[action.type];

    if (typeof actionReducer === "function") {
      return actionReducer(state, action.payload);
    }

    return state;
  };
}
/**
 * Composes any number of reducers of a state type TState into a single reducer of that state type
 *
 * @param reducers
 */

function composeReducers() {
  for (var _len = arguments.length, reducers = new Array(_len), _key = 0; _key < _len; _key++) {
    reducers[_key] = arguments[_key];
  }

  return function (state, action) {
    return reducers.reduce(function (nextState, reducer) {
      return reducer(nextState, action);
    }, state);
  };
}
/**
 *  @deprecated temporary alias for createReducer
 * this alias will be removed in v3.x
 */

var handleActions = createReducer;
/**
 * Registers a reducer handler for one or many actions
 *
 * @param action
 * @param reducer
 */

function reduce(actions, reducer) {
  return Array.isArray(actions) ? actions.reduce(function (acc, action) {
    return merge(acc, action.reduce(reducer));
  }, {}) : actions.reduce(reducer);
}
/**
 * Registers a reducer handler for one or many actions
 *
 * @param action
 * @param reducer
 */

function reduceP(actions, reducer) {
  return Array.isArray(actions) ? actions.reduce(function (acc, action) {
    return merge(acc, action.reduceP(reducer));
  }, {}) : actions.reduceP(reducer);
}
/**
 * Registers a reducer handler for one or many actions
 *
 * @param action
 * @param reducer
 */

function fold(actions, reducer) {
  return Array.isArray(actions) ? actions.reduce(function (acc, action) {
    return merge(acc, action.fold(reducer));
  }, {}) : actions.fold(reducer);
}
/**
 * Registers a reducer handler for one or multiple actions
 *
 * @template TState - the reducer state
 * @template TActions
 *
 * @param actions - a single action creator or multiple action creators
 * @param reducer - a `PartialActionFolder`: TPayload -> TState -> TState
 */

function foldP(actions, reducer) {
  return Array.isArray(actions) ? actions.reduce(function (acc, action) {
    return merge(acc, action.foldP(reducer));
  }, {}) : actions.foldP(reducer);
}
/**
 *  @deprecated temporary alias for `reduce`
 * this alias will be removed in v3.x
 */

var match = reduce;
/**
 *  @deprecated temporary alias for `foldP`
 * this alias will be removed in v3.x
 */

var matchF = foldP;

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var toUpperSnakeCase = function toUpperSnakeCase(s) {
  return s.replace(/(([a-z])([A-Z]))/g, function (_ref) {
    var a = _ref[0],
        b = _ref[1];
    return a + "_" + b;
  }).toUpperCase();
};

/**
 * returns an action-creator function
 *
 * @param type - the action identifier, must be unique
 * @param namespace - optional namespace string to be prepended to the type
 */

function createAction(type, namespace) {
  var $type = toUpperSnakeCase(namespace ? namespace + "/" + type : type);

  var baseActionCreator = function baseActionCreator(payload, options) {
    return {
      error: options ? options.error : undefined,
      meta: options ? options.meta : undefined,
      payload: payload,
      type: $type
    };
  };

  var extensions = {
    type: $type,
    reduce: function reduce(handler) {
      var _ref;

      return _ref = {}, _ref[$type] = handler, _ref;
    },
    reduceP: function reduceP(handler) {
      var _ref2;

      return _ref2 = {}, _ref2[$type] = uncurryN(2, handler), _ref2;
    },
    fold: function fold(handler) {
      var _ref3;

      return _ref3 = {}, _ref3[$type] = flip(handler), _ref3;
    },
    foldP: function foldP(handler) {
      var _ref4;

      return _ref4 = {}, _ref4[$type] = flip(uncurryN(2, handler)), _ref4;
    }
  };
  return Object.assign(baseActionCreator, extensions);
}
/**
 * returns a composite action-creator with nested action-creators for request, success and failure
 *
 * @param type - the action identifier, must be unique
 * @param namespace - optional namespace string to be prepended to the type
 */

function createAsyncAction(type, namespace) {
  var baseAction = createAction(type, namespace);
  var extensions = {
    request: createAction(type + "_REQUEST", namespace),
    success: createAction(type + "_SUCCESS", namespace),
    failure: createAction(type + "_FAILURE", namespace),
    cancel: createAction(type + "_CANCEL", namespace)
  };
  return Object.assign(baseAction, extensions);
}
var CreateActionsAPI = function CreateActionsAPI() {};

CreateActionsAPI.action = function () {
  return function (type, namespace) {
    return createAction(type, namespace);
  };
};

CreateActionsAPI.asyncAction = function () {
  return function (type, namespace) {
    return createAsyncAction(type, namespace);
  };
};

function createActions() {
  var namespace = arguments.length === 1 ? undefined : arguments.length <= 0 ? undefined : arguments[0];
  var actionsContructor = arguments.length === 1 ? arguments.length <= 0 ? undefined : arguments[0] : arguments.length <= 1 ? undefined : arguments[1];
  var defs = actionsContructor(CreateActionsAPI);
  return Object.entries(defs).reduce(function (actionCreatorMap, _ref5) {
    var _extends2;

    var key = _ref5[0],
        factory = _ref5[1];
    return _extends({}, actionCreatorMap, (_extends2 = {}, _extends2[key] = factory(key, namespace), _extends2));
  }, {});
}

/**
 * Higher order saga for handling common remote resource fetching and error handling routines
 *
 * @param asyncAction
 * @param asyncHandler
 * @param hooks
 */

function apiWorkerFactory(asyncAction, asyncHandler, hooks) {
  return /*#__PURE__*/regeneratorRuntime.mark(function sagaWorker(action) {
    var result;
    return regeneratorRuntime.wrap(function sagaWorker$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.prev = 0;
            _context.next = 3;
            return effects.put(asyncAction.request());

          case 3:
            if (!(typeof action.payload === "undefined")) {
              _context.next = 9;
              break;
            }

            _context.next = 6;
            return effects.call(asyncHandler);

          case 6:
            _context.t0 = _context.sent;
            _context.next = 12;
            break;

          case 9:
            _context.next = 11;
            return effects.call(asyncHandler, action.payload);

          case 11:
            _context.t0 = _context.sent;

          case 12:
            result = _context.t0;

            if (!(hooks && hooks.onSuccess)) {
              _context.next = 18;
              break;
            }

            _context.next = 16;
            return effects.fork(hooks.onSuccess, result, action);

          case 16:
            _context.next = 20;
            break;

          case 18:
            _context.next = 20;
            return effects.put(asyncAction.success(result));

          case 20:
            _context.next = 31;
            break;

          case 22:
            _context.prev = 22;
            _context.t1 = _context["catch"](0);

            if (!(hooks && hooks.onFailure)) {
              _context.next = 29;
              break;
            }

            _context.next = 27;
            return effects.fork(hooks.onFailure, _context.t1, action);

          case 27:
            _context.next = 31;
            break;

          case 29:
            _context.next = 31;
            return effects.put(asyncAction.failure(_context.t1));

          case 31:
          case "end":
            return _context.stop();
        }
      }
    }, sagaWorker, null, [[0, 22]]);
  });
}

/**
 * Applies a transform function recursively to each leaf in a tree
 *
 * @param transform - a function to be applied to each leaf
 * @param tree - a tree-like object with deeply nested props
 */

function transformTree(transform, tree) {
  return mapObjIndexed(function (value, key) {
    return typeof value === "object" ? transformTree(transform, value) : transform(value, key);
  }, tree);
}
/**
 * Verifies whether the `target` has all the properties in `props`
 *
 * @param props
 * @param target
 */

function hasOwnProps(props, target) {
  var hasOwnProp = function hasOwnProp(prop) {
    return Object.prototype.hasOwnProperty.call(target, prop);
  };

  return props.every(hasOwnProp);
}

function isAsyncActionCreator(action) {
  return hasOwnProps(["request", "success", "failure", "cancel"], action);
}

var toDispatcher = function toDispatcher(dispatch) {
  return function (action) {
    var baseDispatcher = redux.compose(dispatch, action);

    if (!isAsyncActionCreator(action)) {
      return baseDispatcher;
    }

    var asyncAction = action;
    var extensions = {
      request: redux.compose(dispatch, asyncAction.request),
      success: redux.compose(dispatch, asyncAction.success),
      failure: redux.compose(dispatch, asyncAction.failure),
      cancel: redux.compose(dispatch, asyncAction.cancel)
    };
    return Object.assign(baseDispatcher, extensions);
  };
};
/**
 * bindActionCreators
 *
 * Monkeypatches dispatch to the actions making them self-dispachable
 *
 * @param actions
 */


var bindActionCreators = function bindActionCreators(actions) {
  return function (dispatch) {
    return {
      actions: transformTree(toDispatcher(dispatch), actions)
    };
  };
};
var applySelectors = function applySelectors(spec) {
  return applySpec(spec);
};
/**
 * connectWithActions
 *
 * Wraps react-redux's "connect" helper, binding dispatch to the action-creators
 *
 * @param actions - object of action-creators
 * @param mapStateToProps -
 * It can either be a function that directly maps the app's State to a component's Props
 * or an object whose keys represent properties in the target component and values are functions
 * that take the application's State and optionally, the connected component's own Props.
 * Those functions then return a derived value that matches the component's contract.
 * Ideally should be used with redux selectors.
 *
 */

var connectWithActions = function connectWithActions(actions, mapStateToProps) {
  var stateToProps = typeof mapStateToProps === "object" ? applySelectors(mapStateToProps) : mapStateToProps;
  return reactRedux.connect(stateToProps, bindActionCreators(actions));
};

/**
 * Return store-bound action-creators.
 *
 * @param unboundActions action-creators generated by "re-reduced/createActions"
 */

function useActions(unboundActions) {
  var dispatch = reactRedux.useDispatch();
  var boundActions = react.useMemo(function () {
    var _bindActionCreators = bindActionCreators(unboundActions)(dispatch),
        actions = _bindActionCreators.actions;

    return actions;
  }, [dispatch, unboundActions]);
  return boundActions;
}
/**
 * Returns the deriver redux state based on a map of state seelctors
 *
 * @param stateMapper either normal mapStateToProps (without ownProps as 2nd parameter) or selectorSpecs
 */

function useReduxState(stateMapper) {
  var stateToProps = react.useMemo(function () {
    return typeof stateMapper === "object" ? // selector
    applySpec(stateMapper) : // mapStateFn
    stateMapper;
  }, [stateMapper]);
  var state = reactRedux.useSelector(stateToProps);
  return state;
}

Object.defineProperty(exports, 'connect', {
  enumerable: true,
  get: function () {
    return reactRedux.connect;
  }
});
Object.defineProperty(exports, 'connectAdvanced', {
  enumerable: true,
  get: function () {
    return reactRedux.connectAdvanced;
  }
});
exports.CreateActionsAPI = CreateActionsAPI;
exports.apiWorkerFactory = apiWorkerFactory;
exports.applySelectors = applySelectors;
exports.bindActionCreators = bindActionCreators;
exports.composeReducers = composeReducers;
exports.connectWithActions = connectWithActions;
exports.createAction = createAction;
exports.createActions = createActions;
exports.createAsyncAction = createAsyncAction;
exports.createReducer = createReducer;
exports.fold = fold;
exports.foldP = foldP;
exports.handleActions = handleActions;
exports.match = match;
exports.matchF = matchF;
exports.reduce = reduce;
exports.reduceP = reduceP;
exports.useActions = useActions;
exports.useReduxState = useReduxState;
//# sourceMappingURL=re-reduced.cjs.development.js.map
